/***************************************************************************
                          kmyirc.cpp  -  description
                             -------------------
    begin                : Son Jun 16 13:43:09 CEST 2002
    copyright            : (C) 2002 by Stephan Hermann
    email                : sh@sourcecode.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmyirc.h"

KMyIRC::KMyIRC(QWidget *parent, const char *name) : QextMdiMainFrm(parent, name)
{
  nickTry=0;
  config=new KMIConfig(this,"config");
  render=new KMIOutputRender(this,"render",config,true);
  initCTCPHandler();
  initCommandHandler();
  ircLogic=new KMILogic(this,"ircLogic",render,ctcpHandlerRoot,commandHandlerRoot);
  networkClient=new KMINetworkClient(this,"networkClient");
  connect(networkClient,SIGNAL(connectedToServer()),this,SLOT(slotConnectionEstablished()));
  connect(networkClient,SIGNAL(processData(const QString &)),ircLogic,SLOT(parseIncomingData(const QString &)));
  connect(ircLogic,SIGNAL(sendCommand(const QString &)),networkClient,SLOT(sendNetworkPacket(const QString &)));
  initMenuBar();
  initWindow();
  initStatusBar();
  // setMenuForSDIModeSysButtons(menuBar);
  isConnected=false;
  chanList=0;
  //if (!config->getGlobals()->noServerListAtStartUp) {
  //  slotDoLogin();
  //}
  
}

KMyIRC::~KMyIRC()
{
  delete config;
  delete ircLogic;
  delete networkClient;
}

/** No descriptions */
void KMyIRC::slotConnectionEstablished(){
  qWarning("Hostname: "+networkClient->getOwnHostname());
  file->setItemEnabled(100,false);
	statusBar()->changeItem(i18n("Connected to ")+connectedServer->serverName,2);
  isConnected=true;
	int mode=0;
	if (config->getGlobals()->isInvisible) {
		mode+=8;
	}
	if (config->getGlobals()->receiveWallOps) {
		mode+=4;
  }
  
  ircLogic->doLogin(config->getGlobals()->mainNickname,config->getGlobals()->userName,mode,connectedServer->serverName,config->getGlobals()->realName);
}
/** No descriptions */
void KMyIRC::slotDoLogin(){
  qDebug("slotDoLogin()");
  KMIConnectDlg *dlg=new KMIConnectDlg(this,"connectDlg");
  dlg->setServerList(config->getServerList());
  if (dlg->exec()) {
    serverHost=dlg->getServerHost();
    serverItem *item=config->getServerItem(serverHost);
    statusBar()->changeItem(i18n("Try to connect to ")+item->serverName,2);
    networkClient->connectToHost(item->serverName,QString("%1").arg(item->portNo));
    connectedServer=item;
/*    while (!isConnected) {
      kapp->processEvents();
    }
    statusBar()->changeItem(i18n("Connected to ")+item->serverName,2);
    int mode=0;
    if (config->getGlobals()->isInvisible) {
      mode+=8;
    }
    if (config->getGlobals()->receiveWallOps) {
      mode+=4;
    }
    ircLogic->doLogin(config->getGlobals()->mainNickname,config->getGlobals()->userName,mode,item->serverName,config->getGlobals()->realName);
    */
  }
  delete dlg;
  //ircLogic->doLogin("__sh__","sh",networkClient->getOwnHostname(),"freenet.de","Whatever you Want");
}

/** No descriptions */
void KMyIRC::initWindow(){
  connect(this,SIGNAL(sendInput(const QString &,QWidget *)),ircLogic,SLOT(parseInputCommands(const QString &,QWidget *)));
  connect(ircLogic,SIGNAL(sendInputToView(const QString &)),this,SLOT(sendOutput(const QString &)));
  connect(ircLogic,SIGNAL(connectionEstablished(const QString &)),this,SLOT(connectionEstablished(const QString &)));
  connect(ircLogic,SIGNAL(sendMotdToView(const QString &,const QString &)),this,SLOT(sendMotdToView(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(channelJoin(const QString &)),this,SLOT(joinChannel(const QString &)));
  connect(ircLogic,SIGNAL(addUserToList(const QString &,const QString &)),this,SLOT(addUserToList(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendOutputToChannel(const QString &,const QString &)),this,SLOT(sendOutputToChannel(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendSetOperatorStatus(const QString &,const QString &)),this,SLOT(sendSetOperatorStatus(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendRemoveOperatorStatus(const QString &,const QString &)),this,SLOT(sendRemoveOperatorStatus(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendSetVoice(const QString &,const QString &)),this,SLOT(sendSetVoice(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendRemoveVoice(const QString &,const QString &)),this,SLOT(sendRemoveVoice(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendSetNoMessagesFromOutside(const QString &,bool)),this,SLOT(sendSetNoMessagesFromOutside(const QString &,bool)));
  connect(ircLogic,SIGNAL(sendSetModerated(const QString &,bool)),this,SLOT(sendSetModerated(const QString &,bool)));
  connect(ircLogic,SIGNAL(setNickName(const QString &)),this,SLOT(setNickName(const QString &)));
  connect(ircLogic,SIGNAL(nickChanged(const QString &,const QString &)),this,SLOT(changeNick(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(closeChannelWindow(const QString &)),this,SLOT(closeChannelWindow(const QString &)));
  connect(ircLogic,SIGNAL(removeUserFromChannel(const QString &,const QString &)),this,SLOT(removeUserFromChannel(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendSetAnonymous(const QString &,bool)),this,SLOT(sendSetAnonymous(const QString &,bool)));
  connect(ircLogic,SIGNAL(sendSetTopicProtection(const QString &,bool)),this,SLOT(sendSetTopicProtection(const QString &,bool)));
  connect(ircLogic,SIGNAL(sendPrivateMessageFrom(const QString &,const QString &,const QString &)),this,SLOT(sendPrivateMessageFrom(const QString &,const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendSetInviteOnly(const QString &,bool)),this,SLOT(sendSetInviteOnly(const QString &,bool)));
  connect(ircLogic,SIGNAL(sendSetQuite(const QString &,bool)),this,SLOT(sendSetQuite(const QString &,bool)));
  connect(ircLogic,SIGNAL(sendSetPrivate(const QString &,bool)),this,SLOT(sendSetPrivate(const QString &,bool)));
  connect(ircLogic,SIGNAL(sendSetSecret(const QString &,bool)),this,SLOT(sendSetSecret(const QString &,bool)));
  connect(ircLogic,SIGNAL(sendSetServerReOp(const QString &,bool)),this,SLOT(sendSetServerReOp(const QString &,bool)));
  connect(ircLogic,SIGNAL(sendChannelLimit(const QString &,bool,const QString &)),this,SLOT(sendChannelLimit(const QString &,bool,const QString &)));
  connect(ircLogic,SIGNAL(sendChannelKey(const QString &,bool,const QString &)),this,SLOT(sendChannelKey(const QString &,bool,const QString &)));
  connect(ircLogic,SIGNAL(sendNickAlreadyInUse()),this,SLOT(sendNickAlreadyInUse()));
  connect(ircLogic,SIGNAL(sendSetTopic(const QString &,const QString &)),this,SLOT(sendSetTopic(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(setQuitMessage(const QString &,const QString &)),this,SLOT(setQuitMessage(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendPrivmsgToChannel(const QString &,const QString &,const QString &)),this,SLOT(sendPrivmsgToChannel(const QString &,const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendAction(const QString &,const QString &,const QString &)),this,SLOT(sendAction(const QString &,const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendNotice(const QString &,const QString &,const QString &)),this,SLOT(sendNotice(const QString &,const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendNickChange(const QString &,const QString &,const QString &)),this,SLOT(sendNickChange(const QString &,const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendUserModeChange(const QString &,const QString &)),this,SLOT(sendUserModeChange(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(removeUserFromChannels(const QString &,const QString &)),this,SLOT(removeUserFromChannels(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendChannelModeChange(const QString &,const QString &)),this,SLOT(sendChannelModeChange(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendErrorMessage(const QString &,const QString &, const QString &)),this,SLOT(sendErrorMessage(const QString &,const QString &, const QString &)));
  connect(ircLogic,SIGNAL(sendNameReply(const QString &)),this,SLOT(sendNameReply(const QString &)));
  connect(ircLogic,SIGNAL(sendMsgToChannel(const QString &,const QString &,const QString &)),this,SLOT(sendMsgToChannel(const QString &,const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendWhois(const QString &,const QString &)),this,SLOT(sendWhois(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendAway(const QString &,const QString &)),this,SLOT(sendAway(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendWho(const QString &,const QString &)),this,SLOT(sendWho(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendUsers(const QString &,const QString &)),this,SLOT(sendUsers(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendKickMessage(const QString &,const QString &)),this,SLOT(sendKickMessage(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendInviteMessage(const QString &,const QString &)),this,SLOT(sendInviteMessage(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendLUserMsg(const QString &,const QString &)),this,SLOT(sendLUserMsg(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendVersionMsg(const QString &,const QString &)),this,SLOT(sendVersionMsg(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendListMsg(const QString &,const QString &,const QString &)),this,SLOT(sendListMsg(const QString &,const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendStatsLinkInfo(const QString &,const QString &)),this,SLOT(sendStatsLinkInfo(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendStatsCommand(const QString &,const QString &)),this,SLOT(sendStatsCommand(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendStatsUptime(const QString &,const QString &)),this,SLOT(sendStatsUptime(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendStatsOLine(const QString &,const QString &)),this,SLOT(sendStatsOLine(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendEndOfStats(const QString &,const QString &)),this,SLOT(sendEndOfStats(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendLinksMsg(const QString &,const QString &)),this,SLOT(sendLinksMsg(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendEndOfLinksMsg(const QString &,const QString &)),this,SLOT(sendEndOfLinksMsg(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendTimeMsg(const QString &,const QString &)),this,SLOT(sendTimeMsg(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendTraceMsg(const QString &,const QString &)),this,SLOT(sendTraceMsg(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendAdminMsg(const QString &,const QString &)),this,SLOT(sendAdminMsg(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendInfoMsg(const QString &,const QString &)),this,SLOT(sendInfoMsg(const QString &,const QString &)));
  connect(ircLogic,SIGNAL(sendServListMsg(const QString &,const QString &)),this,SLOT(sendServListMsg(const QString &,const QString &)));
}
/** No descriptions */
void KMyIRC::slotQuitApplication(){
  qDebug("in slotQuitApplication()");
  config->writeGlobals();
  // if (networkClient->isConnected()) {
  //   qDebug("Sending QUIT");
  //   networkClient->sendNetworkPacket("QUIT :Client Quitting\n");
  //   
  //   while (!networkClient->isConnectionClosed()) {
  //     kapp->processEvents();
  //   }
  // }
  kapp->quit();
}
/** No descriptions */
void KMyIRC::connectionEstablished(const QString &title){
  qDebug("Connection Established to "+title);
  // Changed Here 
  channels.insert(title,new KMIChannelWindow(title,NULL,title,false,false,config));
  addWindow(channels[title]);
  connect(channels[title],SIGNAL(sendInput(const QString &,QWidget *)),SIGNAL(sendInput(const QString &,QWidget *)));
  connect(ircLogic,SIGNAL(setNickName(const QString &)),this,SLOT(setNickName(const QString &)));
  resize(channels[title]->size());
  //
  // bug no.
  //
  channels[title]->maximize(true);
  config->setMyNick(ircLogic->getOwnNick());
}
/** No descriptions */
void KMyIRC::initMenuBar(){
  //menuBar=new KMenuBar(this,"menuBar");
  KActionCollection *actionCollection=new KActionCollection(this,"");
  file=new KPopupMenu(this);
  file->insertItem(i18n("&Connect..."),this,SLOT(slotDoLogin()),0,100);
  file->insertSeparator();
  file->insertItem(i18n("Quit"),this,SLOT(slotQuitApplication()));
  // KAction *quitIt=new KAction(i18n("quitIt"),0,this,SLOT(slotQuitApplication()),actionCollection,"AS-BLOCK");
  // quitIt->plug(file);
  channel=new KPopupMenu(this);
  channel->insertItem(i18n("List channels..."),this,SLOT(slotListChannels()));
  misc=new KPopupMenu(this);
  misc->insertItem(i18n("Raw IRC message window"),this,SLOT(slotRawIRCWindow()));
  configMenu=new KPopupMenu(this);
  configMenu->insertItem(i18n("&IRC-Server... "),this,SLOT(slotConfigServer()));
  configMenu->insertItem(i18n("&CTCP-Replies... "),this,SLOT(slotConfigCTCPReturns()));
  configMenu->insertItem(i18n("&Popupmenu Editor... "),this,SLOT(slotConfigPopupEdit()));
  menuBar()->insertItem(i18n("&File"),file);
  menuBar()->insertItem(i18n("C&hannels"),channel);
  menuBar()->insertItem(i18n("&Configuraton"),configMenu);
  menuBar()->insertItem(i18n("&Misc"),misc);
  KHelpMenu *help=new KHelpMenu(this,KGlobal::instance()->aboutData());
  if (!isFakingSDIApplication()) {
    menuBar()->insertItem(i18n("&Window"), windowMenu());
  }
  setMenuForSDIModeSysButtons(menuBar());
  menuBar()->insertItem(i18n("&Help"),help->menu());
  //menuBar()->show();
}
/** No descriptions */
void KMyIRC::sendOutput(const QString &windowTitle,const QString &message){
  qDebug("sendOutput window,message");
  KMIChannelWindow *temp=channels[windowTitle];
  if (temp!=0 && !temp->isServer()) {
    temp->output(message);
  }
}
/** No descriptions */
void KMyIRC::sendOutput(const QString &message){
  qDebug("only SendOutput");
  KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
  if (temp!=0 && !temp->isServer()) {
    temp->output(message);
  }
}
/** No descriptions */
void KMyIRC::joinChannel(const QString &title){
  qDebug("Joining Channel: "+title);
  if (channels.find(title)==0) {
    channels.insert(title,new KMIChannelWindow(title,NULL,title,true,false,config));
    addWindow(channels[title]);
    connect(channels[title],SIGNAL(sendInput(const QString &,QWidget *)),SIGNAL(sendInput(const QString &,QWidget *)));
    channels[title]->changeNickName(ircLogic->getOwnNick());
  } else {
    channels[title]->activate();
  }
}
/** No descriptions */
void KMyIRC::addUserToList(const QString &channel,const QString &user){
  
  kdDebug() << "addUserToList() Sendername: " << sender()->name() << "   Channelname: " << channel << " User: " << user << "\n";
  if (channels.find(channel)!=0) {
    channels[channel]->slotAddUserToList(user);
  }
}
/** No descriptions */
void KMyIRC::sendOutputToChannel(const QString &channel,const QString &message){
  kdDebug() << "in SendOutputToChannel! Channel ==> " << channel << "   : message ==> " << message << "\n";
  if (channels.find(channel)!=0) {
    // Output Message Handling
    channels[channel]->output(message);
  }
}
/** No descriptions */
void KMyIRC::sendSetOperatorStatus(const QString &channel,const QString &user){
  if (channels.find(channel)!=0) {
    channels[channel]->addOp(user);
  }
}
/** No descriptions */
void KMyIRC::sendRemoveOperatorStatus(const QString &channel,const QString &user){
  if (channels.find(channel)!=0) {
    channels[channel]->removeOp(user);
  }
}
/** No descriptions */
void KMyIRC::sendSetVoice(const QString &channel,const QString &user){
  if (channels.find(channel)!=0) {
    channels[channel]->addVoice(user);
  }
}
/** No descriptions */
void KMyIRC::sendRemoveVoice(const QString &channel,const QString &user){
  if (channels.find(channel)!=0) {
    channels[channel]->removeVoice(user);
  }
}
/** No descriptions */
void KMyIRC::sendSetNoMessagesFromOutside(const QString &channel,bool on){
  if (channels.find(channel)!=0) {
    channels[channel]->toggleNoMessagesFromOutside(on);
  }
}
/** No descriptions */
void KMyIRC::sendSetModerated(const QString &channel,bool on){
  if (channels.find(channel)!=0) {
    channels[channel]->toggleModerate(on);
  }
}
/** No descriptions */
void KMyIRC::setNickName(const QString &nick){
  kdDebug() << "in SetNickName";
   QDictIterator<KMIChannelWindow> it( channels ); // See QDictIterator
    for( ; it.current(); ++it ) {
        it.current()->changeNickName(nick);
    }
}
/** No descriptions */
void KMyIRC::changeNick(const QString &old,const QString &news){
   QDictIterator<KMIChannelWindow> it( channels ); // See QDictIterator
   for( ; it.current(); ++it ) {
     if (!it.current()->isServer() && !it.current()->isPrivateMsgWin()) {
       it.current()->changeChannelNick(old,news);
     }
   }
}
/** No descriptions */
void KMyIRC::closeChannelWindow(const QString &channel){
  KMIChannelWindow *temp;
  if (channels.find(channel)!=0) {
    temp=channels.find(channel);
	temp->setParted(true);
    closeWindow(temp);
  }
}
/** No descriptions */
void KMyIRC::removeUserFromChannel(const QString &channel,const QString &user){
  if (channels.find(channel)!=0) {
    channels[channel]->removeUser(user);
  }
}
/** No descriptions */
void KMyIRC::sendSetAnonymous(const QString &channel,bool on){
  if (channels.find(channel)!=0) {
    channels[channel]->setAnonymous(on);
  }
}
/** No descriptions */
void KMyIRC::sendSetTopicProtection(const QString &channel,bool on){
  if (channels.find(channel)!=0) {
    channels[channel]->setTopicProtection(on);
  }
}
/** No descriptions */
void KMyIRC::closeWindow (QextMdiChildView *pWnd, bool layoutTaskBar)  {
  qDebug("In closeWindow event");
  qDebug("pWnd->Name() == %s",pWnd->name());
  if (QString(pWnd->name())!="ChannelList" && QString(pWnd->name())!="rawIRC") {
    KMIChannelWindow *temp=(KMIChannelWindow *)pWnd;
    if (temp!=0) {
      qDebug("temp->getChannelName() == "+temp->getChannelName());
      if (!temp->isPrivateMsgWin() && !temp->isServer()) {
        qDebug("in Part");
		if (!temp->isParted()) {
        	networkClient->sendNetworkPacket("PART "+temp->getChannelName()+"\n");
		}
        channels.take(temp->getChannelName());
      } else if (temp->isServer() && !temp->isPrivateMsgWin()) {
        qDebug("QUIT CloseWindoww");
        channels.take(temp->getChannelName());
        networkClient->sendNetworkPacket("QUIT :Client Quitting\n");
      }
    }
    QextMdiMainFrm::closeWindow(pWnd,layoutTaskBar);
  } else {
    QextMdiMainFrm::closeWindow(pWnd,layoutTaskBar);
    chanList=0;
  }    
}
/** No descriptions */
void KMyIRC::slotConfigServer(){
  KMIConfigDialog *dlg=new KMIConfigDialog(this,"configDialog");
  dlg->setConfigParams(*config->getGlobals());
  dlg->setServerList(config->getServerList());
  dlg->setMessageTextList(config->getMessageTextList());
  dlg->setColors(config->getColorList());
  if (dlg->exec()) {
    dlg->getConfigParams(config->getGlobals());
    dlg->updateServerList();
    dlg->updateMessageTextList();
    dlg->updateColorList();
    config->setServerList(dlg->getServerList());
    config->setMessageTextList(dlg->getMessageTextList());
    config->setColorList(dlg->getColorList());
  }
  delete dlg;
}
/** No descriptions */
void KMyIRC::sendPrivateMessageFrom(const QString &to,const QString &from,const QString &message){
  if (channels.find(to)!=0) {
  	// CHANGED !!!!!!
     channels[to]->output(message);
  } else {
    channels.insert(to,new KMIChannelWindow(from,NULL,from,false,true,config));
    addWindow(channels[to]);
    connect(channels[to],SIGNAL(sendInput(const QString &,QWidget *)),SIGNAL(sendInput(const QString &,QWidget *)));
    channels[to]->changeNickName(ircLogic->getOwnNick());
    channels[to]->output(message);
  }
}
/** No descriptions */
void KMyIRC::sendSetInviteOnly(const QString &channel,bool on){
  if (channels.find(channel)!=0) {
    channels[channel]->toggleInviteOnly(on);
  }
}
/** No descriptions */
void KMyIRC::sendSetQuite(const QString &channel,bool on){
  if (channels.find(channel)!=0) {
    channels[channel]->toggleQuite(on);
  }
}
/** No descriptions */
void KMyIRC::sendSetPrivate(const QString &channel,bool on){
  if (channels.find(channel)!=0) {
    channels[channel]->togglePrivate(on);
  }

}
/** No descriptions */
void KMyIRC::sendSetSecret(const QString &channel,bool on){
  if (channels.find(channel)!=0) {
    channels[channel]->toggleSecret(on);
  }

}
/** No descriptions */
void KMyIRC::sendSetServerReOp(const QString &channel,bool on){
  if (channels.find(channel)!=0) {
    channels[channel]->toggleServerReOp(on);
  }

}
/** No descriptions */
void KMyIRC::sendChannelLimit(const QString &channel,bool on,const QString &limit){
  if (channels.find(channel)!=0) {
    channels[channel]->toggleChannelLimit(on,limit);
  }
}
/** No descriptions */
void KMyIRC::sendChannelKey(const QString &channel,bool on,const QString &key){
  if (channels.find(channel)!=0) {
    channels[channel]->toggleChannelKey(on,key);
  }
}
/** No descriptions */
void KMyIRC::sendNickAlreadyInUse(){
  nickTry++;
  if (nickTry>0) {
    int mode=0;
    if (config->getGlobals()->isInvisible) {
      mode+=8;
    }
    if (config->getGlobals()->receiveWallOps) {
      mode+=4;
    }
    ircLogic->doLogin(config->getGlobals()->altNickname,config->getGlobals()->userName,mode,connectedServer->serverName,config->getGlobals()->realName);
  }
}
/** No descriptions */
void KMyIRC::sendSetTopic(const QString &channel,const QString &topic){
  if (channels.find(channel)!=0) {
    channels[channel]->setTopic(topic);
  }
}
/** No descriptions */
void KMyIRC::setQuitMessage(const QString &nick,const QString &message){
   QDictIterator<KMIChannelWindow> it( channels ); // See QDictIterator
   for( ; it.current(); ++it ) {
     if (it.current()->findNickInChannel(nick)!=0 && !it.current()->isServer() && !it.current()->isPrivateMsgWin()) {
       it.current()->output(message);
       removeUserFromChannel(it.current()->getChannelName(),nick);
     }
   }
}
/** No descriptions */
void KMyIRC::slotConfigCTCPReturns(){
  KMICtcpReplies *dlg=new KMICtcpReplies(this,"ctcpReplies");
  dlg->setCtcpReplies(config->getCtcpReplies());
  if (dlg->exec()) {
    config->setCtcpReplies(dlg->getCtcpReplies());
    delete  ctcpHandlerRoot;
    initCTCPHandler();
  }
  delete dlg;
}


/** No descriptions */
void KMyIRC::sendMotdToView(const QString &channel,const QString &message){
  KMIChannelWindow *temp=channels[channel];
  if (temp!=0) {
    temp->output(message);
  }  
}
/** No descriptions */
void KMyIRC::sendPrivmsgToChannel(const QString &channel,const QString &from,const QString &message){
  if (channels.find(channel)!=0) {
    if (!channels[channel]->isServer()) {
      channels[channel]->output(message);
      changeTabButton(channels[channel]);
    }
  }
}
/** No descriptions */
void KMyIRC::sendAction(const QString &channel,const QString &from,const QString &message){
  if (channels.find(channel)!=0) {
    channels[channel]->output(message);
  }
}
/** No descriptions */
void KMyIRC::sendNotice(const QString &channel,const QString &from,const QString &message){
  if (channels.find(channel)!=0) {
    channels[channel]->output(message);
  }
}
/** No descriptions */
void KMyIRC::sendNickChange(const QString &oldNick,const QString &newNick,const QString &message) {
   QDictIterator<KMIChannelWindow> it( channels ); // See QDictIterator
   bool found=false;
   for( ; it.current(); ++it ) {
     if (it.current()->findNickInChannel(newNick)!=0) {
       it.current()->output(message);
       found=true;
     } 
   }
   if (!found) {
     // channels[ircLogic->getServerIdent()]->output(render->renderOutput(KMIOutputRender::NICK,oldNick,newNick));
   }
}
/** No descriptions */
void KMyIRC::sendUserModeChange(const QString &channel,const QString &message) {
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(message);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(message);
  }
}
/** No descriptions */
void KMyIRC::removeUserFromChannels(const QString &channel,const QString &user){
  QDictIterator<KMIChannelWindow> it( channels ); // See QDictIterator
  for( ; it.current(); ++it ) {
    if (it.current()->findNickInChannel(user)!=0) {
      channels[channel]->removeUser(user);      
    }
  }
}
/** No descriptions */
void KMyIRC::sendChannelModeChange(const QString &channel,const QString &message){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(message);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(message);
  }
}
/** No descriptions */
void KMyIRC::sendErrorMessage(const QString &channel,const QString &nick, const QString &message){
  if (channels.find(channel)!=0) {
    channels[channel]->output(message);
  } else if (channel=="!!!actualWindow") {
    // Check for Actual Window
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(message);
  }
}
/** No descriptions */

void KMyIRC::resizeEvent ( QResizeEvent *e){
   QextMdiMainFrm::resizeEvent( e);
   setSysButtonsAtMenuPosition();
}
/** No descriptions */
void KMyIRC::sendNameReply(const QString &message){
  qDebug("bin hier");
  KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
  temp->output(message);
}
/** No descriptions */
void KMyIRC::sendMsgToChannel(const QString &channel,const QString &nick,const QString &message){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(message);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(message);
  }
}
/** No descriptions */
void KMyIRC::initCTCPHandler(){
  ctcpHandlerRoot=new KMICTCPHandler(this,"ctcpHandlerRoot",config);
  (void) new KMICTCPActionHandler(ctcpHandlerRoot);
  (void) new KMICTCPVersionHandler(ctcpHandlerRoot,VERSION,"(C) 2002 by Stephan Hermann");
  (void) new KMICTCPSourceHandler(ctcpHandlerRoot,"http://www.kde-coder.de/users/sh/kmyirc-0.1.tar.gz");
  for (s_msgText *i=config->getCtcpReplies()->first();i;i=config->getCtcpReplies()->next()) {
    (void) new KMICTCPCommonHandler(ctcpHandlerRoot,i->msgType,i->msgText);
  }
  
}
/** No descriptions */
void KMyIRC::initCommandHandler(){
  commandHandlerRoot=new KMIInputCommand(this,"inputCommandHandler",render);
  (void) new KMICommandNotice(commandHandlerRoot);
  (void) new KMICommandMsg(commandHandlerRoot);
  (void) new KMICommandJoin(commandHandlerRoot);
  (void) new KMICommandNick(commandHandlerRoot);
  (void) new KMICommandMode(commandHandlerRoot);
  (void) new KMICommandWhois(commandHandlerRoot);
  (void) new KMICommandWho(commandHandlerRoot);
  (void) new KMICommandAway(commandHandlerRoot);
  (void) new KMICommandUsers(commandHandlerRoot);
  (void) new KMICommandKick(commandHandlerRoot);
  (void) new KMICommandPart(commandHandlerRoot);
  (void) new KMICommandNames(commandHandlerRoot);
  (void) new KMICommandInvite(commandHandlerRoot);
  (void) new KMICommandQuit(commandHandlerRoot);
  (void) new KMICommandTopic(commandHandlerRoot);
  (void) new KMICommandLuser(commandHandlerRoot);
  (void) new KMICommandVersion(commandHandlerRoot);
  (void) new KMICommandList(commandHandlerRoot);
  (void) new KMICommandStats(commandHandlerRoot);
  (void) new KMICommandLinks(commandHandlerRoot);
  (void) new KMICommandTime(commandHandlerRoot);
  (void) new KMICommandTrace(commandHandlerRoot);
  (void) new KMICommandAdmin(commandHandlerRoot);
  (void) new KMICommandInfo(commandHandlerRoot);
  (void) new KMICommandServList(commandHandlerRoot);
  (void) new KMICommandWhowas(commandHandlerRoot);
  (void) new KMICommandCtcp(commandHandlerRoot);
  (void) new KMICommandLeave(commandHandlerRoot);
}
/** No descriptions */
void KMyIRC::sendWhois(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if(channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }
}
/** No descriptions */
void KMyIRC::sendAway(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }    
}
/** No descriptions */
void KMyIRC::sendWho(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    if (QString(activeWindow()->name())!="ChannelList") {
      KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
      temp->output(msg);
    } else {
      QDictIterator<KMIChannelWindow> it( channels ); // See QDictIterator
      for( ; it.current(); ++it ) {
        if (it.current()->isServer()) {
          it.current()->output(msg);
        }
     }
   }      
  }
}
/** No descriptions */
void KMyIRC::sendUsers(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }
}
/** No descriptions */
void KMyIRC::sendKickMessage(const QString &channel,const QString &message){
  if (channels.find(channel)!=0) {
    channels[channel]->output(message);
  }
}
/** No descriptions */
void KMyIRC::sendInviteMessage(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }
}
/** No descriptions */
void KMyIRC::initStatusBar(){
  statusBar()->insertItem("Test",2,0,true);
}
/** No descriptions */
void KMyIRC::changeTabButton(KMIChannelWindow *channel){
  if (mdiMode()==QextMdi::ChildframeMode) {
    if (activeWindow()!=channel) {
      QextMdiTaskBarButton *btn=m_pTaskBar->getButton(channel);
      if (btn!=0) {
        btn->setPaletteForegroundColor(QColor(255,0,0));
      }
    }
  }
}
/** No descriptions */
void KMyIRC::activateView(QextMdiChildView *pWnd){
  if (mdiMode()==QextMdi::ChildframeMode) {
    QextMdiTaskBarButton *btn=m_pTaskBar->getButton(pWnd);
    if (btn!=0) {
      btn->setPaletteForegroundColor(QColorGroup::ButtonText);
    }
  }
  QextMdiMainFrm::activateView(pWnd);
}
/** No descriptions */
void KMyIRC::closeEvent(QCloseEvent *e){
  qDebug("in closeEvent");
  // slotQuitApplication();
  config->writeGlobals();
  kapp->quit();
}
/** No descriptions */
KMIConfig * KMyIRC::getCfg(){
  return config;
}
/** No descriptions */
void KMyIRC::sendLUserMsg(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }
}
/** No descriptions */
void KMyIRC::sendVersionMsg(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }
}
/** No descriptions */
void KMyIRC::slotListChannels(){
  if (chanList==0) {
    chanList=new KMIChannelList(NULL,"ChannelList");
    addWindow(chanList);
    connect(chanList,SIGNAL(sendInput(const QString &,QWidget *)),networkClient,SLOT(sendNetworkPacket(const QString &)));
  }
}
/** No descriptions */
void KMyIRC::sendListMsg(const QString &channel,const QString &nou,const QString &descr){
  slotListChannels();
  chanList->addChannel(channel,nou,descr);
}
/** No descriptions */
void KMyIRC::sendStatsLinkInfo(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }
}
/** No descriptions */
void KMyIRC::sendStatsCommand(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }
}
/** No descriptions */
void KMyIRC::sendStatsUptime(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }
}
/** No descriptions */
void KMyIRC::sendStatsOLine(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }
}
/** No descriptions */
void KMyIRC::sendEndOfStats(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }
}
/** No descriptions */
void KMyIRC::sendLinksMsg(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }
}
/** No descriptions */
void KMyIRC::sendEndOfLinksMsg(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }
}
/** No descriptions */
void KMyIRC::sendTimeMsg(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }  
}
/** No descriptions */
void KMyIRC::sendTraceMsg(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }
}
/** No descriptions */
void KMyIRC::sendAdminMsg(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }

}
/** No descriptions */
void KMyIRC::sendInfoMsg(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }

}
/** No descriptions */
void KMyIRC::sendServListMsg(const QString &channel,const QString &msg){
  if (!channel.isEmpty()) {
    if (channels.find(channel)!=0) {
      channels[channel]->output(msg);
    }
  } else {
    KMIChannelWindow *temp=(KMIChannelWindow *)activeWindow();
    temp->output(msg);
  }
}
/** No descriptions */
void KMyIRC::slotConfigPopupEdit(){
	KMIPopupEdit *dlg=new KMIPopupEdit(this,"popupEditDlg",config->getAliasList());
  if (dlg->exec()) {
  	config->setAliasList(dlg->getAliasList());
  }
  delete dlg;  
}

void KMyIRC::slotRawIRCWindow()
{
	KMIRawIRCWindow *raw=new KMIRawIRCWindow(NULL,"rawIRC");
	connect(networkClient,SIGNAL(sendOutputToView(const QString &)),raw,SLOT(output(const QString &)));
	addWindow(raw);
}
