/***************************************************************************
                          kmiconfig.cpp  -  description
                             -------------------
    begin                : Son Jun 23 2002
    copyright            : (C) 2002 by Stephan Hermann
    email                : sh@sourcecode.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmiconfig.h"

KMIConfig::KMIConfig(QObject *parent,const char *name):QObject(parent,name){
  listServer=new serverList();
  listColors=new colorList();
  listMessageText=new textList();
  listCtcpReplies=new textList();
  aliasList=new l_aliasCmdList();
  // cfg=new KSimpleConfig("kmiconfigurationrc");
  cfg=KGlobal::config();
  globals=new s_Globals;
  readGlobals();
}
KMIConfig::~KMIConfig(){
  writeGlobals();
  delete globals;
  delete listServer;
  delete cfg;
}
/** No descriptions */
void KMIConfig::readGlobals(){
  QStringList intern=cfg->groupList();
  cfg->setGroup("GLOBALS");
  globals->mainNickname=cfg->readEntry("MainNickname","KMIUser");
  globals->altNickname=cfg->readEntry("AltNickname","KMIUser_");
  globals->realName=cfg->readEntry("RealName","Foo Bar");
  globals->userName=cfg->readEntry("UserNAme","foobar");
  globals->isInvisible=cfg->readBoolEntry("Invisible",false);
  globals->receiveWallOps=cfg->readBoolEntry("WallOps",false);
  globals->noServerListAtStartUp=cfg->readBoolEntry("StartUpServerList",false);
  globals->showTime=cfg->readUnsignedNumEntry("ShowTime",0);
  globals->highlightPhrase=cfg->readEntry("HighlightPhrase","");
  
  // Reading servers
  for ( QStringList::Iterator it = intern.begin(); it != intern.end(); ++it ) {
    if ((*it).contains("ServerName ")) {
      QString grpName=(*it).mid(QString("ServerName ").length());
      cfg->setGroup(*it);
      QMap<QString,QString> keys=cfg->entryMap(*it);
      QMap<QString,QString>::Iterator it1;
      serverItem *temp=new serverItem;
      for (it1=keys.begin();it1!=keys.end();++it1) {
        if (it1.key()=="Name") {;
          temp->name=it1.data();
        } else if (it1.key()=="ServerName") {
          temp->serverName=it1.data();
        } else if (it1.key()=="PortNo") {
          temp->portNo=it1.data().toUInt();
        } else if (it1.key()=="Group") {
          temp->group=it1.data();
        }
      }
      listServer->append(temp);
    }
  }
  // Reading the colors
  cfg->setGroup("Colors");
  QMap<QString,QString> keys1=cfg->entryMap("Colors");
  for (QMap<QString,QString>::Iterator it2=keys1.begin();it2!=keys1.end();++it2) {
    s_msgColors *col=new s_msgColors;
    col->msgType=it2.key();
    col->msgColor=QColor(it2.data());
    qWarning("msgType: "+col->msgType+"\n");
    qWarning("msgColor: "+it2.data()+"\n");
    listColors->append(col);
  }
  // reading text
  cfg->setGroup("Message Text");
  QMap<QString,QString> keys2=cfg->entryMap("Message Text");
  for (QMap<QString,QString>::Iterator it3=keys2.begin();it3!=keys2.end();++it3) {
    s_msgText *tt=new s_msgText;
    tt->msgType=it3.key();
    tt->msgText=it3.data();
    qWarning("msgType: "+it3.key()+"\n");
    qWarning("msgColor: "+it3.data()+"\n");    
    listMessageText->append(tt);
  }
  // reading ctcpreplies
  cfg->setGroup("CTCP Replies");
  QMap<QString,QString> keys3=cfg->entryMap("CTCP Replies");
  for (QMap<QString,QString>::Iterator it3=keys3.begin();it3!=keys3.end();++it3) {
    s_msgText *tt=new s_msgText;
    tt->msgType=it3.key();
    tt->msgText=it3.data();
    listCtcpReplies->append(tt);
  }

  cfg->setGroup("Alias Commands");
  QMap<QString,QString> keys4=cfg->entryMap("Alias Commands");
  for (QMap<QString,QString>::Iterator it3=keys4.begin();it3!=keys4.end();++it3) {
    s_aliasList *tt=new s_aliasList;
    tt->aliasName=it3.key();
    QString data=it3.data();
    QStringList strList=QStringList::split(";",data);
    QStringList::Iterator it99=strList.at(1);
    tt->aliasCommands=QString(*it99);
    it99=strList.at(0);
    tt->ofType=QString(*it99).toInt();
    aliasList->append(tt);
  }
  
}
/** No descriptions */
void KMIConfig::writeGlobals(){
  deleteGroups();
  cfg->setGroup("GLOBALS");
  cfg->writeEntry("MainNickname",globals->mainNickname);
  cfg->writeEntry("AltNickname",globals->altNickname);
  cfg->writeEntry("RealName",globals->realName);
  cfg->writeEntry("UserName",globals->userName);
  cfg->writeEntry("Invisible",globals->isInvisible);
  cfg->writeEntry("WallOps",globals->receiveWallOps);
  cfg->writeEntry("StartUpServerList",globals->noServerListAtStartUp);
  cfg->writeEntry("ShowTime",globals->showTime);
  cfg->writeEntry("HighlightPhrase",globals->highlightPhrase);
  // Writing Servers
  for(serverItem *i=listServer->first();i;i=listServer->next()) {
    cfg->setGroup(QString("ServerName %1").arg(i->name));
    cfg->writeEntry("Group",i->group);
    cfg->writeEntry("Name",i->name);
    cfg->writeEntry("ServerName",i->serverName);
    cfg->writeEntry("PortNo",i->portNo);    
  }
  // Writing colors
  cfg->setGroup("Colors");
  for(s_msgColors *i=listColors->first();i;i=listColors->next()) {
    cfg->writeEntry(i->msgType,i->msgColor.name());
  }
  // writing text
  cfg->setGroup("Message Text");
  for (s_msgText *i=listMessageText->first();i;i=listMessageText->next()) {
    cfg->writeEntry(i->msgType,i->msgText);
  }
  // writing ctcpReplies
  cfg->setGroup("CTCP Replies");
  for (s_msgText *i=listCtcpReplies->first();i;i=listCtcpReplies->next()) {
    cfg->writeEntry(i->msgType,i->msgText);
  }
  cfg->setGroup("Alias Commands");
  for (s_aliasList *i=aliasList->first();i;i=aliasList->next()) {
    cfg->writeEntry(i->aliasName,QString("%1;").arg(i->ofType)+i->aliasCommands);
  }
  cfg->sync();
}
/** No descriptions */
s_Globals * KMIConfig::getGlobals(){
  return globals;
}
/** No descriptions */
void KMIConfig::setGlobals(s_Globals *glb){
  globals=glb;
}
/** No descriptions */
serverList *KMIConfig::getServerList(){
  return listServer;
}
/** No descriptions */
void KMIConfig::setServerList(serverList *list){
  listServer=list;
}
/** No descriptions */
void KMIConfig::deleteGroups(){
  cfg->setGroup("");
  QStringList intern=cfg->groupList();
  for ( QStringList::Iterator it = intern.begin(); it != intern.end(); ++it ) {
    cfg->deleteGroup(*it);
  }
}
/** No descriptions */
serverItem * KMIConfig::getServerItem(const QString &server){
  for ( serverItem *item = listServer->first(); item; item = listServer->next() ) {
    if (item->serverName==server) {
      return item;
    }
  }
  return 0;
}
/** No descriptions */
colorList * KMIConfig::getColorList(){
  return listColors;
}
/** No descriptions */
void KMIConfig::setColorList(colorList *list){
  listColors=list;
}
/** No descriptions */
textList * KMIConfig::getMessageTextList(){
  return listMessageText;
}
/** No descriptions */
void KMIConfig::setMessageTextList(textList *list){
  listMessageText=list;
}
/** No descriptions */
const QString & KMIConfig::getMyNick(){
  return myNick;
}
/** No descriptions */
void KMIConfig::setMyNick(const QString &nick){
  myNick=nick;
}
/** No descriptions */
QString KMIConfig::findMessageText(const QString &msgType){
  for (s_msgText *i=listMessageText->first();i;i=listMessageText->next()) {
    if (i->msgType==msgType) {
      return i->msgText;
    }
  }
  return "not found";
}
/** No descriptions */
QString KMIConfig::findColor(const QString &msgType){
  for (s_msgColors *i=listColors->first();i;i=listColors->next()) {
    if (i->msgType==msgType) {
      return i->msgColor.name();
    }
  }
  return QString("#000000");
}
/** No descriptions */
void KMIConfig::setCtcpReplies(textList *list){
  listCtcpReplies=list;
}
/** No descriptions */
textList * KMIConfig::getCtcpReplies(){
  return listCtcpReplies;
}
/** No descriptions */
l_aliasCmdList *KMIConfig::getAliasList(){
  return aliasList;
}
/** No descriptions */
QString KMIConfig::findAlias(const QString &alias){
  for (s_aliasList *i=aliasList->first();i;i=aliasList->next()) {
    if (i->aliasName==alias) {
      return i->aliasCommands;
    }
  }
  return QString("");
}

s_aliasList* KMIConfig::findAliasPtr(const QString & name)
{
	for (s_aliasList *i=aliasList->first();i;i=aliasList->next()) {
		if (i->aliasName==name) {
			return i;
		}
	}
	return NULL;

}

void KMIConfig::setAliasList(l_aliasCmdList *list)
{
	// Copy all new items to aliasList
  aliasList->clear();
  for (s_aliasList *tt=list->first();tt;tt=list->next()) {
    s_aliasList *item=new s_aliasList;
    item->aliasName=tt->aliasName;
    item->aliasCommands=tt->aliasCommands;
    item->ofType=tt->ofType;
    aliasList->append(item);
  }  
}
