/***************************************************************************
                          kmilogic.cpp  -  description
                             -------------------
    begin                : Son Jun 16 2002
    copyright            : (C) 2002 by Stephan Hermann
    email                : sh@sourcecode.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../logic/kmilogic.h"

KMILogic::KMILogic(QObject *parent,const char *name,KMIOutputRender *_renderOutput,KMICTCPHandler *ctcp,KMIInputCommand *commandHandler):QObject(parent,name){
  ctcpHandler=ctcp;
  command=commandHandler;
  renderOutput=_renderOutput;
  connect(command,SIGNAL(sendCommand(const QString &)),SIGNAL(sendCommand(const QString &)));
  connect(command,SIGNAL(sendMsgToChannel(const QString &,const QString &,const QString &)),SIGNAL(sendMsgToChannel(const QString &,const QString &,const QString &)));
                          //<server>|<nick> <command> <params>
  returnCodes=new QRegExp("^:(.+) (\\d+) (.+) ");
  returnCodes->setMinimal(true);
  returnCommands=new QRegExp("^:(.*) (\\D+) (.+)");
  returnCommands->setMinimal(true);
  beginMotd=false;
  beginWhois=false;
  inJoinProcess=false;
  inLogin=false;
  inUsers=false;

}
KMILogic::~KMILogic(){
  delete returnCodes;
}
/** No descriptions */
void KMILogic::parseIncomingData(const QString &line){
  // qDebug(line);
  QString line1=line;
  KMIIrcMessage *testMessage=new KMIIrcMessage(this,"ircMessageParser");
  testMessage->parseMessage(line1);
  if (line.contains(QRegExp("^PING"))) {
    QString tt=line;
    tt.remove(0,5);
    sendCommand("PONG "+tt+"\n");
    qDebug("PONG "+tt);
  }
  int pos=returnCodes->search(line);
  if (pos>=0) {
    if (returnCodes->cap(2).length()==3) {
      serverIdent=returnCodes->cap(1);
      if (!parseReturnCodes(returnCodes->cap(2).toUInt(),line)) {
        sendInputToView(QString("Return Code not understood or not implemented.\n Please send this to sh@sourcecode.de\nReturn Code: %1").arg(returnCodes->cap(2).toUInt()));
      }
    }
  }
  pos=returnCommands->search(line);
  if (pos>=0) {
    if (!parseReturnCommands(returnCommands->cap(1),returnCommands->cap(2),line)) {
      // sendinputtoview
    }
  }
}
/** No descriptions */
void KMILogic::doLogin(const QString &nickNName,const QString &userName,int mode,const QString &server, const QString &realName){
  QString command;
  myNick=nickNName;
  command="NICK "+nickNName+"\n";
  command+="USER "+userName+" "+QString("%1").arg(mode)+" "+server+" :"+realName+"\n";
  inLogin=true;
  sendCommand(command);
}



/** No descriptions */
void KMILogic::parseInputCommands(const QString &line,QWidget *sender){
  QDict<QString> replacements;
  replacements.setAutoDelete(true);
  KMIChannelWindow *temp=(KMIChannelWindow *)sender;
  if (line.find("/",false)==0) {
    QString line1=line;
    line1.remove(0,1);
    QString command1=line1.left(line1.find(' '));
    line1.replace(0,line1.find(' ')+1,"");
    if (command1.lower()=="me") {
      KMICTCPParser *ctcp=new KMICTCPParser(this);
      QString ctcpMsg=ctcp->getCTCPMessage("ACTION "+line1);
      sendCommand("PRIVMSG "+temp->getChannelName()+" :"+ctcpMsg+"\n");
      QString msg=ctcpHandler->ctcpOutput(myNick,temp->getChannelName(),ctcpMsg);
      // sendPrivateMessageFrom(getNickName(myNick).lower(),getNickName(from),msg);
      replacements.clear();
      replacements.insert("nick",new QString(myNick));
      replacements.insert("msg",new QString(msg));
      sendAction(temp->getChannelName().lower(),myNick,renderOutput->renderOutput("ME",replacements));

      delete ctcp;
    } else {
      command->getCommandLine(temp->getChannelName().lower(),getNickName(myNick),line);
      qDebug(command1+" "+line1+"\n");
      // sendCommand(command1+" "+line1+"\n");
    }
  } else {
    if (!temp->isServer()) {
      qDebug("hello");
      sendCommand(":"+myNick+" PRIVMSG "+temp->getChannelName()+" :"+line+"\n");
      replacements.clear();
      replacements.insert("nick",new QString(myNick));
      replacements.insert("msg", new QString(line));
      sendPrivmsgToChannel(temp->getChannelName().lower(),myNick,renderOutput->renderOutput("PRIVMSG",replacements));
    }
    // sendInputToView("<b>&lt;"+myNick+"&gt;</b> "+line);
  }
}
/** No descriptions */
bool KMILogic::parseReturnCodes(const uint &code,const QString &message){
  bool isHandled;
  int pos;
  QDict<QString> replacements;
  replacements.setAutoDelete(true);
  isHandled=false;
  QString line=QString(message).replace(QRegExp("\r\\n$"),"");
  QString from=returnCodes->cap(1);
  line=line.replace(0,QString(":"+returnCodes->cap(1)+" "+returnCodes->cap(2)+" "+returnCodes->cap(3)+" ").length(),"");
  switch(code) {
    case RPL_WELCOME: { // After Connection
      isHandled=true;
      QRegExp *reg1=new QRegExp("^:Welcome to the Internet Relay Network (.*)$");
      // reg1->setMinimal(true);
      pos=reg1->search(line);
      if (pos>=0) {
        qDebug("Nick Identd ======> "+reg1->cap(1));
      }
      inLogin=false;
      connectionEstablished(returnCodes->cap(1).lower());
      setNickName(myNick);
      replacements.clear();
      replacements.insert("msg",new QString(line.remove(line.find(":"),1)));
      sendMotdToView(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      delete reg1;
      break;
    }
    case RPL_WHOWASUSER:
    case RPL_WHOISUSER: {
      qDebug("in Whois begin");
      isHandled=true;
      beginWhois=true;
      // Cracking Line
      QString nick=line.left(line.find(" "));
      line.remove(0,line.find(" ")+1);
      QString user=line.left(line.find(" "));
      line.remove(0,line.find(" ")+1);
      QString host=line.left(line.find(" "));
      line.remove(0,line.find(" ")+1);
      QString realName=line.mid(line.find(":")+1);
      replacements.clear();
      QString email;
      email.sprintf("%s@%s",user.latin1(),host.latin1());
      email.replace(QRegExp("[~+]"),"");
      replacements.insert("nick",new QString(nick));
      replacements.insert("user",new QString(user));
      replacements.insert("host",new QString(host));
      replacements.insert("realname",new QString(realName));
      replacements.insert("email",new QString(email));
      sendWhois("",renderOutput->renderOutput(code,replacements));
      qDebug("tempOutput: "+tempOutput);
      break;
    }
    case RPL_WHOISCHANNELS: {
      qDebug("RPL_WHOISCHANNELS");
      isHandled=true;
      if (beginWhois) {
        QString nick=line.left(line.find(":")-1);
        QString channels=line.mid(line.find(":")+1);
        replacements.clear();
        replacements.insert("nick",new QString(nick));
        replacements.insert("channel",new QString(channels));
        sendWhois("",renderOutput->renderOutput(code,replacements));
      }
      break;
    }
    case RPL_WHOISSERVER: {
      isHandled=true;
      if (beginWhois) {
        qDebug("RPL_WHOISSERVER: line == "+line);
        QString nick=line.left(line.find(" "));
        line.remove(0,line.find(" "));
        QString server=line.left(line.find(":")-1);
        QString serverDescr=line.mid(line.find(":")+1);
        replacements.clear();
        replacements.insert("nick",new QString(nick));
        replacements.insert("server",new QString(server));
        replacements.insert("descr",new QString(serverDescr));
        sendWhois("",renderOutput->renderOutput(code,replacements));
      }
      break;
    }
    case RPL_WHOISIDLE: {
      // RFC Implementation "<nick> <integer> :seconds idle"
      // NON RFC Compliant Enhancement: <nick> <integer> <integer> :seconds idle, signon time
      qDebug("RPL_WHOISIDLE line == "+line);
      if (beginWhois) {
        QString nick=line.left(line.find(" "));
        line.remove(0,line.find(" ")+1);
        QString idle=line.left(line.find(" "));
        replacements.clear();
        replacements.insert("nick",new QString(nick));
        replacements.insert("idle",new QString(idle));
        sendWhois("",renderOutput->renderOutput(code,replacements));
      }                     
      isHandled=true;
      break;
    }
    case RPL_WHOISOPERATOR: {
      if (beginWhois) {
        replacements.clear();
        replacements.insert("nick", new QString(line.left(line.find(" "))));
        sendWhois("",renderOutput->renderOutput(code,replacements));
      }
      isHandled=true;
      break;
    }
    case RPL_ENDOFWHOIS: {
      isHandled=true;
      if (beginWhois) {
        beginWhois=false;
        replacements.clear();
        replacements.insert("nick",new QString(line.left(line.find(" "))));
        sendWhois("",renderOutput->renderOutput(code,replacements));
      }
      break;
    }
    case RPL_WHOREPLY: {
      QString channel=line.left(line.find(" "));
      line.remove(0,line.find(" ")+1);
      QString user=line.left(line.find(" "));
      line.remove(0,line.find(" ")+1);
      QString host=line.left(line.find(" "));
      line.remove(0,line.find(" ")+1);
      QString server=line.left(line.find(" "));
      line.remove(0,line.find(" ")+1);
      QString nick=line.left(line.find(" "));
      line.remove(0,line.find(" ")+1);
      QString flag=line.left(line.find(" "));
      line.remove(0,line.find(" ")+1);
      QString operflag=line.left(line.find(" "));
      line.remove(0,line.find(" ")+1);
      QString opflag=line.left(line.find(" "));
      line.remove(0,line.find(" ")+1);
      QString hopcount=line.mid(line.find(":"));
      line.remove(0,line.find(" ")+1);
      QString realName=line.left(line.find(" "));
      replacements.clear();
      replacements.insert("channel",new QString(channel));
      replacements.insert("user",new QString(user));
      replacements.insert("host",new QString(host));
      replacements.insert("server",new QString(server));
      replacements.insert("nick",new QString(nick));
      replacements.insert("hgflag",new QString(flag));
      replacements.insert("operflag",new QString(operflag));
      replacements.insert("opflag",new QString(opflag));
      replacements.insert("hopcount",new QString(hopcount));
      replacements.insert("realname",new QString(realName));
      sendWho("",renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    case RPL_ENDOFWHO: {
      replacements.clear();
      sendWho("",renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    case RPL_AWAY: {
      QString nick=line.left(line.find(' '));
      QString reason=line.mid(line.find(":")+1);
      qDebug("In RPL_AWAY: Line =="+line);
      qDebug("reason: "+reason);
      replacements.clear();
      replacements.insert("nick",new QString(nick));
      replacements.insert("reason",new QString(reason));
      if (beginWhois) {
        sendWhois("",renderOutput->renderOutput(code,replacements));
      } else {
        sendAway("",renderOutput->renderOutput(code,replacements));
      }
    	isHandled=true;
    	break;
    }
    case RPL_UNAWAY: {
      replacements.clear();
    	sendAway("",renderOutput->renderOutput(code,replacements));
    	isHandled=true;
    	break;
    }
    case RPL_NOWAWAY: {
      replacements.clear();
      // #warning IMPLEMENT ME
      sendAway("",renderOutput->renderOutput(code,replacements));
    	isHandled=true;
    	break;
    }
    case RPL_USERSSTART: {
      replacements.clear();      
      // #warning IMPLEMENTME USERSSTART
      sendUsers("",renderOutput->renderOutput(code,replacements));
      break;
    }
    case RPL_USERS: {
      replacements.clear();
      // #warning IMPLEMENTME USERS
      sendUsers("",renderOutput->renderOutput(code,replacements));
      break;
    }
    case RPL_ENDOFUSERS: {
      replacements.clear();
      // #warning IMPLEMENTME ENDOFUSERS
      sendUsers("",renderOutput->renderOutput(code,replacements));
      break;
    }
    case RPL_NOUSERS: {
      replacements.clear();
      // #warning IMPLEMENTME NOUSERS
      sendUsers("",renderOutput->renderOutput(code,replacements));
      break;
    }
    case RPL_MOTDSTART: { // MotD Begin
      qDebug("in motd begin");
      replacements.clear();
      replacements.insert("msg",new QString(line.remove(line.find(":"),1)));
      sendMotdToView(serverIdent.lower(),renderOutput->renderOutput(code,replacements));      
      beginMotd=true;
      isHandled=true;
      break;
    }
    case RPL_MOTD: { // MotD 
      if (beginMotd) {
        replacements.clear();
        replacements.insert("msg",new QString(line.remove(line.find(":"),1)));
        sendMotdToView(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
        // motd+=line+"\n";
        isHandled=true;
      }
      break;
    }
    case RPL_ENDOFMOTD: {
      if (beginMotd) { // Motd End
        // sendMotdToView(serverIdent,motd);
        replacements.clear();
        replacements.insert("msg",new QString(line.remove(line.find(":"),1)));
        sendMotdToView(serverIdent.lower(),renderOutput->renderOutput(code,replacements));        
        beginMotd=false;
        isHandled=true;
      }
      break;
    }
    case RPL_NAMREPLY: {
      QRegExp *reg1=new QRegExp("^(\\=|\\*|\\@) (.*) :(.*)$");
      reg1->setMinimal(true);
      pos=reg1->search(line);
      if (pos>=0) {
        if (inJoinProcess) {
          qWarning("Channeltype:  "+reg1->cap(1));
          qWarning("Nicknames :" +reg1->cap(3));
          QStringList names=QStringList::split(" ",reg1->cap(3));
          for ( QStringList::Iterator it = names.begin(); it != names.end(); ++it ) {
            qDebug("Hallo");
            addUserToList(reg1->cap(2).lower(),*it);
          }
        } else {
            replacements.clear();
            replacements.insert("channel",new QString(reg1->cap(2)));
            replacements.insert("user",new QString(reg1->cap(3)));
            sendNameReply(renderOutput->renderOutput(code,replacements));
        }
      }
      delete reg1;
      isHandled=true;
      break;
    }
    case RPL_ENDOFNAMES: {
      if (inJoinProcess) {
        inJoinProcess=false;
      }
      isHandled=true;
      break;
    }
    case RPL_TOPIC: {
      qDebug("RPL_TOPIC: "+line);
      // QString channel=line.
      QString channelName=line.left(line.find(":")-1);
      QString topic=line.mid(line.find(":")+1);
      sendSetTopic(channelName.lower(),topic);
      isHandled=true;
      break;
    }
    case RPL_INVITING: {
      QString nick=line.left(line.find(" "));
      QString channelName=line.mid(line.find(" ")+1);
      replacements.clear(),
      replacements.insert("invitewho",new QString(nick));
      replacements.insert("invitewhere",new QString(channelName));
      sendInviteMessage("",renderOutput->renderOutput("INVITEOTHER",replacements));
      isHandled=true;
      break;
    }
    case RPL_CHANNELMODEIS: {
      QString channelName=line.left(line.find(" "));
      line.remove(0,line.find(" ")+1);
      QString modes=line.left(line.find(" "));
      line.remove(0,line.find(" ")+1);
      QString nicks=line.left(line.length());
      if (channelName.lower()==myNick.lower()) {
        nicks="";
      }
      setChannelModes("MODE",from,channelName,modes,nicks);
      
      isHandled=true;
      break;
    }
    // LUSERS
    case RPL_LUSERCLIENT: { // ":There are <integer> users and <integer> services on <integer> servers"
      QRegExp regtemp("There are (\\d+) users and (\\d+) services on (\\d+) servers");
      int pos=regtemp.search(line);
      if (pos>=0) {
          replacements.clear();
          replacements.insert("users",new QString(regtemp.cap(1)));
          replacements.insert("services",new QString(regtemp.cap(2)));
          replacements.insert("servers",new QString(regtemp.cap(3)));
          sendLUserMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      }
      isHandled=true;
      break;
    }    
    case RPL_LUSEROP: { // "<integer> :operator(s) online"
      QRegExp regtemp("(\\d+) :operator(s) online");
      int pos=regtemp.search(line);
      if (pos>=0) {
        replacements.clear();
        replacements.insert("operator", new QString(regtemp.cap(1)));
        sendLUserMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      }
      isHandled=true;
      break;    
    }
    case RPL_LUSERUNKNOWN: { // "<integer> :unknown connection(s)"
      QRegExp regtemp("(\\d+) :unknown connection(s)");
      int pos=regtemp.search(line);
      if (pos>=0) {
        replacements.clear();
        replacements.insert("connections",new QString(regtemp.cap(1)));
        sendLUserMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      }
      isHandled=true;
      break;    
    }
    case RPL_LUSERCHANNELS: {   // "<integer> :channels formed"
      QRegExp regtemp("(\\d+) :channels formed");
      int pos=regtemp.search(line);
      if (pos>=0){
        replacements.clear();
        replacements.insert("channels",new QString(regtemp.cap(1)));
        sendLUserMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      }
      isHandled=true;
      break;    
    }
    case RPL_LUSERME: { // ":I have <integer> clients and <integer> servers"
      QRegExp regtemp(".*(\\d+).*(\\d+)\\w+");
      int pos=regtemp.search(line);
      if (pos>=0) {
        replacements.clear();
        replacements.insert("clients",new QString(regtemp.cap(1)));
        replacements.insert("servers",new QString(regtemp.cap(2)));
        sendLUserMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      }
      isHandled=true;
      break;    
    }
    case RPL_VERSION: {
      QRegExp regtemp("(.*)\\.(.*) (.*) :(.*)");
      int pos=regtemp.search(line);
      if (pos>=0) {
        replacements.clear();
        replacements.insert("version",new QString(regtemp.cap(1)));
        replacements.insert("debug",new QString(regtemp.cap(2)));
        replacements.insert("server",new QString(regtemp.cap(3)));
        replacements.insert("comments",new QString(regtemp.cap(4)));
      }
      sendVersionMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    case RPL_LIST: {
      QString channelName=line.left(line.find(" "));
      line.remove(0,line.find(" ")+1);
      QString numberOfUsers=line.left(line.find(" "));
      line.remove(0,line.find(":")+1);
      QString description=line;
      replacements.clear();
      replacements.insert("channel",new QString(channelName));
      replacements.insert("numberofusers",new QString(numberOfUsers));
      replacements.insert("description",new QString(description));
      sendListMsg(channelName,numberOfUsers,description);
      isHandled=true;
      break;
    }
    case RPL_STATSLINKINFO: {
      replacements.clear();
      replacements.insert("line",new QString(line));
      sendStatsLinkInfo(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    case RPL_STATSCOMMANDS: {
      replacements.clear();
      replacements.insert("line",new QString(line));
      sendStatsCommand(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    case RPL_ENDOFSTATS: {
      replacements.clear();
      sendEndOfStats(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    case RPL_STATSUPTIME: {
      replacements.clear();
      replacements.insert("line",new QString(line));
      sendStatsUptime(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    case RPL_STATSOLINE: {
      replacements.clear();
      replacements.insert("line",new QString(line));
      sendStatsOLine(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    case RPL_LINKS: {
      QString mask=line.left(line.find(" "));
      line.remove(0,line.find(" ")+1);
      QString server=line.left(line.find(" "));
      line.remove(0,line.find(":")+1);
      QString hopCount=line.left(line.find(" "));
      line.remove(0,line.find(" ")+1);
      QString serverInfo=line;
      replacements.clear();
      replacements.insert("mask",new QString(mask));
      replacements.insert("server",new QString(server));
      replacements.insert("hopcount",new QString(hopCount));
      replacements.insert("info", new QString(serverInfo));
      sendLinksMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;      
    }
    case RPL_ENDOFLINKS: {
      QString mask=line.left(line.find(" "));
      line.remove(0,line.find(":")+1);
      replacements.clear();
      replacements.insert("mask",new QString (mask));
      sendEndOfLinksMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    // TIME
    case RPL_TIME: {
      line.remove(0,line.find(":")+1);
      QString times=line;
      replacements.clear();
      replacements.insert("time",new QString(times));
      sendTimeMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }

    // TRACE
    case RPL_TRACELINK: {
      replacements.clear();
      replacements.insert("tracelink",new QString(line));
      sendTraceMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    case RPL_TRACECONNECTING: {
      replacements.clear();
      replacements.insert("traceconnecting",new QString(line));
      sendTraceMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));    
      isHandled=true;
      break;
    }
    case RPL_TRACEHANDSHAKE: {
      replacements.clear();
      replacements.insert("tracehandshake",new QString(line));
      sendTraceMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));    
      isHandled=true;
      break;
    }
    case RPL_TRACEUNKNOWN: {
      replacements.clear();
      replacements.insert("traceunknown",new QString(line));
      sendTraceMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));    
      isHandled=true;
      break;
    }
    case RPL_TRACEOPERATOR: {
      replacements.clear();
      replacements.insert("traceoperator",new QString(line));
      sendTraceMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));    
      isHandled=true;
      break;
    }
    case RPL_TRACEUSER: {
      replacements.clear();
      replacements.insert("traceuser",new QString(line));
      sendTraceMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));    
      isHandled=true;
      break;
    }
    case RPL_TRACESERVER: {
      replacements.clear();
      replacements.insert("traceserver",new QString(line));
      sendTraceMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));    
      isHandled=true;
      break;
    }
    case RPL_TRACESERVICE: {
      replacements.clear();
      replacements.insert("traceservice",new QString(line));
      sendTraceMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));    
      isHandled=true;
      break;
    }
    case RPL_TRACENEWTYPE: {
      replacements.clear();
      replacements.insert("tracenewtype",new QString(line));
      sendTraceMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));    
      isHandled=true;
      break;
    }
    case RPL_TRACECLASS: {
      replacements.clear();
      replacements.insert("traceclass",new QString(line));
      sendTraceMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));    
      isHandled=true;
      break;
    }
    case RPL_TRACERECONNECT: {
      replacements.clear();
      replacements.insert("tracereconnect",new QString(line));
      sendTraceMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));    
      isHandled=true;
      break;
    }
    case RPL_TRACELOG: {
      replacements.clear();
      replacements.insert("tracelog",new QString(line));
      sendTraceMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));    
      isHandled=true;
      break;
    }
    case RPL_TRACEEND: {
      replacements.clear();
      replacements.insert("traceend",new QString(line));
      sendTraceMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    // ADMIN
    case RPL_ADMINME: {
      replacements.clear();
      replacements.insert("adminme",new QString(line));
      sendAdminMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    case RPL_ADMINLOC1: {
      replacements.clear();
      replacements.insert("adminloc1",new QString(line));
      sendAdminMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    case RPL_ADMINLOC2: {
      replacements.clear();
      replacements.insert("adminloc2",new QString(line));
      sendAdminMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    case RPL_ADMINEMAIL: {
      replacements.clear();
      replacements.insert("adminemail",new QString(line));
      sendAdminMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    // INFO
    case RPL_INFO: {
      replacements.clear();
      replacements.insert("info",new QString(line));
      sendInfoMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    case RPL_ENDOFINFO: {
      replacements.clear();
      replacements.insert("info",new QString(line));
      sendInfoMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));    
      isHandled=true;
      break;
    }
    // SERVLIST
    case RPL_SERVLIST: {
      replacements.clear();
      replacements.insert("servlist",new QString(line));
      sendServListMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    case RPL_SERVLISTEND: {
      replacements.clear();
      replacements.insert("servlistend",new QString(line));
      sendServListMsg(serverIdent.lower(),renderOutput->renderOutput(code,replacements));
      isHandled=true;
      break;
    }
    //
    // Errors
    //
    case ERR_NICKNAMEINUSE: {
      qDebug("line == "+line);
      if (inLogin) {
        sendNickAlreadyInUse();
      } else {
        // output error
        line.replace(0,line.find(":")+1,"");
        sendInputToView(line.left(line.length()));
      }
      isHandled=true;
      break;
    }
    case ERR_NOSUCHNICK: {
      kdDebug() << "ERR_NOSUCHNICK" << "\n";
      QString noNick=line.left(line.find(":")-1);
      QString reason=line.mid(line.find(":")+1);
      sendErrorMessage("!!!actualWindow",noNick,reason);
      isHandled=true;
      break;
    }
    case ERR_NONICKNAMEGIVEN: {
      QString reason=line.mid(line.find(":")+1);
      sendErrorMessage("!!!actualWindow","",reason);
      isHandled=true;
      break;
    }
    case ERR_ERRONEUSNICKNAME: {
      sendErrorMessage("!!!actualWindow",line.left(line.find(":")-1),line.mid(line.find(":")+1));
      isHandled=true;
      break;
    }
    case ERR_NICKCOLLISION: {
      sendErrorMessage("!!!actualWindow",line.left(line.find(":")-1),line.mid(line.find(":")+1));
      isHandled=true;
      break;
    }
    case ERR_UNAVAILRESOURCE: {
      sendErrorMessage("!!!actualWindow",line.left(line.find(":")-1),line.mid(line.find(":")+1));
      isHandled=true;
      break;
    }
    case ERR_NEEDMOREPARAMS: {
      sendErrorMessage("!!!actualWindow",line.left(line.find(":")-1),line.mid(line.find(":")+1));
      isHandled=true;
      break;
    }
    case ERR_ALREADYREGISTRED: {
      sendErrorMessage("!!!actualWindow",line.left(line.find(":")),line.mid(line.find(":")+1));
      isHandled=true;
      break;
    }
    case ERR_RESTRICTED: {
      sendErrorMessage("!!!actualWindow",line.left(line.find(":")),line.mid(line.find(":")+1));
      isHandled=true;
      break;
    }
    case ERR_NOSUCHSERVER: {
      sendErrorMessage("!!!actualWindow",line.left(line.find(":")-1),line.mid(line.find(":")+1));
      isHandled=true;
      break;
    }
    case ERR_NOTOPLEVEL:
    case ERR_WILDTOPLEVEL:
    case ERR_BADMASK:
    case ERR_NOSUCHCHANNEL:
    case ERR_CANNOTSENDTOCHAN:
    case ERR_TOOMANYCHANNELS:
    case ERR_WASNOSUCHNICK:
    case ERR_TOOMANYTARGETS:
    case ERR_UNKNOWNCOMMAND:
    case ERR_NOADMININFO:
    case ERR_NOTONCHANNEL:
    case ERR_NOLOGIN:
    case ERR_KEYSET:
    case ERR_CHANNELISFULL:
    case ERR_UNKNOWNMODE:
    case ERR_INVITEONLYCHAN:
    case ERR_BANNEDFROMCHAN:
    case ERR_BADCHANNELKEY:
    case ERR_BADCHANMASK:
    case ERR_NOCHANMODES:
    case ERR_CHANOPRIVSNEEDED:
    case ERR_NOSUCHSERVICE: {
      sendErrorMessage("!!!actualWindow",line.left(line.find(":")-1),line.mid(line.find(":")+1));
      isHandled=true;
      break;    
    }
    case ERR_NOTEXTTOSEND:
    case ERR_NORECIPIENT:
    case ERR_NOMOTD:
    case ERR_FILEERROR:
    case ERR_SUMMONDISABLED:
    case ERR_USERSDISABLED:
    case ERR_NOTREGISTERED:
    case ERR_NOPERMFORHOST:
    case ERR_PASSWDMISMATCH:
    case ERR_NOPRIVILEGES:
    case ERR_CANTKILLSERVER:
    case ERR_UNIQOPPRIVSNEEDED:
    case ERR_NOOPERHOST:
    case ERR_UMODEUNKNOWNFLAG:
    case ERR_USERSDONTMATCH:
    case ERR_YOUREBANNEDCREEP:
    case ERR_NOORIGIN: {
      sendErrorMessage("!!!actualWindow",line.left(line.find(":")),line.mid(line.find(":")+1));
      isHandled=true;
      break;    
    }
    case ERR_USERONCHANNEL:
    case ERR_USERNOTINCHANNEL: {
      QString _nick=line.left(line.find(" "));
      QString _channel=line.mid(line.find(" ")+1,line.find(":")-1);
      QString _reason=line.mid(line.find(":")+1);
      sendErrorMessage(_channel,_nick,_reason);
      isHandled=true;
      break;
    }
    default:
      break;
    }
    return isHandled;
}

/** No descriptions */
bool KMILogic::parseReturnCommands(const QString &from,const QString &command,const QString &message){
  bool isHandled;
  isHandled=false;
  QDict<QString> replacements;
  replacements.setAutoDelete(true);
  QString line=QString(message).replace(QRegExp("\r\\n$"),"");
  line=line.replace(0,QString(":"+from+" "+command+" ").length(),"");
  if (command=="JOIN") { 
    QString channelname=line.mid(1);
    qDebug("JOIN COMMAND Returned: channel == "+channelname);
    if (from.left(from.find("!"))==myNick) { // And User == self
      qDebug("joining "+channelname);
      inJoinProcess=true;
      channelJoin(channelname.lower());
      sendCommand("MODE "+channelname.lower()+"\n");
      // sendCommand("TOPIC "+channelname.lower()+"\n");
    } else { // Other user joined
      qDebug("User "+from+" joined channel "+channelname);
      addUserToList(channelname.lower(),from.left(from.find("!")));
      replacements.clear();
      replacements.insert("user",new QString(from));
      replacements.insert("channel",new QString(channelname));
      sendOutputToChannel(channelname.lower(),renderOutput->renderOutput(command,replacements));
    }
    isHandled=true;
  }
  if (command=="MODE") {
    QString channelName=line.left(line.find(' '));
    line=line.replace(0,line.find(' ')+1,"");
    QString modes=line.left(line.find(' '));
    line=line.replace(0,line.find(' ')+1,"");
    QString nicks=line.left(line.length());
    if (channelName.lower()==myNick.lower()) {
      modes="";
      parseUserModes(from,nicks);
    } else {
      setChannelModes(command,from,channelName,modes,nicks);
    }
    
  isHandled=true;
  }
  if (command=="PRIVMSG") {
    QString channelname=line.mid(0,line.find(":")-1);
    line.replace(0,channelname.length()+1,"");
    QString message=line.mid(1);
    // Hier nochmals denken
    QString msg=ctcpHandler->ctcpOutput(from,channelname,message);
    if (msg!="") {
      if (msg.left(msg.find(" ")).lower()=="command") {
        sendCommand(msg.mid(msg.find(" ")+1)+"\n");
      } else {
        if (channelname.lower()==myNick.lower()) {
          replacements.clear();
          replacements.insert("nick",new QString(getNickName(from)));
          replacements.insert("msg", new QString(msg));
          sendAction(getNickName(myNick).lower(),getNickName(from),renderOutput->renderOutput("ME",replacements));
        } else {
          replacements.clear();
          replacements.insert("nick",new QString(getNickName(from)));
          replacements.insert("msg", new QString(msg));        
          sendAction(channelname.lower(),getNickName(from),renderOutput->renderOutput("ME",replacements));
        }
      }
    } else if (msg=="UNKNOWN") {
      sendInputToView("*** No CTCP Action Handler found for this");
    } else {
      if (channelname.lower()==myNick.lower()) {
        replacements.clear();
        replacements.insert("nick",new QString(getNickName(from)));
        replacements.insert("msg",new QString(message));
        sendPrivateMessageFrom(getNickName(from).lower(),getNickName(from),renderOutput->renderOutput(command,replacements));
      } else {
        replacements.clear();
        replacements.insert("nick",new QString(getNickName(from)));
        replacements.insert("msg",new QString(message));      
        sendPrivmsgToChannel(channelname.lower(),getNickName(from),renderOutput->renderOutput(command,replacements));
        // sendOutputToChannel(channelname.lower(),"<B>&lt;"+from.left(from.find("!"))+"&gt;</B> "+message+"<br>");
      }
    }
    isHandled=true;
  }
  if (command=="QUIT") {
    QString reason=line.mid(1);
    
    setQuitMessage(from.left(from.find("!")),i18n("<font color=\"%1\">*** User <B>%2</B> quits (%3)").arg("#0000ff").arg(from).arg(reason));
    // removeUserFromChannels("",getNickName(from));
    isHandled=true;
  }
  if (command=="NICK") {
    QString newNick=line.mid(1,line.length()-1);
    if (getNickName(from)==myNick) {
      myNick=newNick;
      setNickName(newNick);
    } 
    nickChanged(getNickName(from),newNick);
    replacements.clear();
    replacements.insert("oldnick",new QString(getNickName(from)));
    replacements.insert("newnick",new QString(newNick));
    sendNickChange(getNickName(from),newNick,renderOutput->renderOutput(command,replacements));
    
    isHandled=true;
  }
  if (command=="PART") {
    QString channelName=line.mid(0,line.find(":")-1);
    QString reason=line.mid(line.find(":")+1);
    if (from.left(from.find("!"))==myNick) {
      closeChannelWindow(channelName);
    } else {
      removeUserFromChannel(channelName,from.left(from.find("!")));
      replacements.clear();
      replacements.insert("user",new QString(from));
      replacements.insert("channel",new QString(channelName));
      replacements.insert("reason",new QString(reason));
      sendOutputToChannel(channelName,renderOutput->renderOutput(command,replacements));
      // i18n("<font color=\"%1\">*** User <B>%2</B> left channel %3 (%4)<br>").arg("#ff0000").arg(from).arg(channelName).arg(reason));
    }
    isHandled=true;
  }
  if (command=="NOTICE") {
    qDebug("in NOTICE");
    QString channelName=line.mid(0,line.find(":")-1);
    QString reason=line.mid(line.find(":")+1);
    KMICTCPParser *parser=new KMICTCPParser(this);
    reason=parser->ctcpDequote(reason);
    replacements.clear();
    replacements.insert("nick",new QString(getNickName(from)));
    replacements.insert("msg",new QString(reason));    
    if (channelName.contains("$")) { // ?? fuer was war denn das ?      
      sendNotice(serverIdent,getNickName(from),renderOutput->renderOutput(command,replacements));
    } else {
      sendNotice(serverIdent,getNickName(from),renderOutput->renderOutput(command,replacements));
    }
    isHandled=true;
  }
  if (command=="TOPIC") {
    qDebug("in TOPIC");
    QString channelName=line.mid(0,line.find(":")-1);
    QString topic=line.mid(line.find(":")+1);
    sendSetTopic(channelName.lower(),topic);
    isHandled=true;
  }
  if (command=="KICK") {
    QString channelName=line.left(line.find(" "));
    line.remove(0,line.find(" ")+1);
    QString nick=line.left(line.find(" "));
    line.remove(0,line.find(" ")+1);
    QString reason=line.mid(line.find(":")+1);
    replacements.clear();
    replacements.insert("from",new QString(getNickName(from)));
    replacements.insert("kickwhom",new QString(getNickName(nick)));
    replacements.insert("channel",new QString(channelName));
    replacements.insert("msg",new QString(reason));
    removeUserFromChannel(channelName.lower(),getNickName(nick));
    sendKickMessage(channelName.lower(),renderOutput->renderOutput(command,replacements));
    isHandled=true;
  }
  if (command=="INVITE") {
    QString nick=line.left(line.find(" "));
    QString channel=line.mid(line.find(":")+1);
    replacements.clear();
    replacements.insert("invitefrom",new QString(getNickName(from)));
    replacements.insert("invitewhere",new QString(channel));
    sendInviteMessage("",renderOutput->renderOutput("INVITEYOU",replacements));
    isHandled=true;
  }
  return isHandled;
}
/** No descriptions */
QString KMILogic::getNickName(const QString &from){
  QString temp=from;
  temp=temp.left(temp.find("!"));
  return temp;
}
/** No descriptions */
const QString & KMILogic::getOwnNick(){
  return myNick;
}
/** No descriptions */
QString KMILogic::parseCtcpMessage(const QString &from,const QString &message){
  QString checkCtcp=message;
  QString returnMessage;
  if (checkCtcp.contains('\x001')) {
    qDebug("Begin CTCP Message");
    checkCtcp.replace(checkCtcp.find('\x001'),1,"");
    if (checkCtcp.contains(QRegExp("^ACTION"))) {
      checkCtcp.replace(0,QString("ACTION ").length(),"");
      checkCtcp.replace(checkCtcp.find('\x001'),1,"");
      returnMessage+=QString("<font color=\"%1\"><B>* %2 %3</B></font><br>").arg("#ff9900").arg(from.left(from.find('!'))).arg(checkCtcp);
    }
  } else {
    returnMessage="";
  }
  return returnMessage;  
}
/** No descriptions */
void KMILogic::setChannelModes(const QString &command,const QString &_from,const QString &_channel,const QString &_modes,const QString &_nicks){
  QString from=_from;
  QDict<QString> replacements;
  QString channelName=_channel;
  QString modes=_modes;
  QString nicks=_nicks;
    bool isSet;

      qDebug("Channel: "+channelName);
      qDebug("Modes: "+modes);
      qDebug("Nicks: "+nicks);
      // +nt+lk 72 dasd
      // +nt-lk etc.

      QStringList list=QStringList::split(" ",nicks);
      if (list.count()>=0) { // Modes with nicks/params
        QStringList::Iterator it=list.begin();
        for (uint i=0;i<modes.length();i++) {
          switch(modes.at(i).latin1()) {
            case '+': {
              isSet=true;
              break;
            }
            case '-': {
              isSet=false;
              break;
            }
            case 'o': {
              if (isSet) {
                qDebug("in+o");
                sendSetOperatorStatus(channelName.lower(),*it);
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("+o"));
                replacements.insert("setremove",new QString("sets"));
                replacements.insert("tonick",new QString(*it));
                sendUserModeChange(channelName.lower(),renderOutput->renderOutput("MODE_USER",replacements));

                // sendOutputToChannel(channelName.lower(),i18n("<font color=\"%1\">*** User <B>%2</B> gives operator status to <B>%3</B></font><br>").arg("#ff0000").arg(from).arg(*it));
                ++it;
              } else {
                sendRemoveOperatorStatus(channelName.lower(),*it);
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("-o"));
                replacements.insert("setremove",new QString("removes"));
                replacements.insert("tonick",new QString(*it));
                sendUserModeChange(channelName.lower(),renderOutput->renderOutput("MODE_USER",replacements));                
                ++it;
              }
              break;
            }
            case 'v': {
              if (isSet) {
                sendSetVoice(channelName.lower(),*it);
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("+v"));
                replacements.insert("setremove",new QString("sets"));
                replacements.insert("tonick",new QString(*it));
                sendUserModeChange(channelName.lower(),renderOutput->renderOutput("MODE_USER",replacements));
                ++it;
              } else {
                sendRemoveVoice(channelName.lower(),*it);
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("-v"));
                replacements.insert("setremove",new QString("removes"));
                replacements.insert("tonick",new QString(*it));
                sendUserModeChange(channelName.lower(),renderOutput->renderOutput("MODE_USER",replacements));
                ++it;
              }
              break;
            }
            case 'b': {
              if (isSet) {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("+b"));
                replacements.insert("setremove",new QString("set"));
                replacements.insert("tonick",new QString(*it));
                sendUserModeChange(channelName.lower(),renderOutput->renderOutput("MODE_USER",replacements));              
                ++it;
              } else {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("-b"));
                replacements.insert("setremove",new QString("removes"));
                replacements.insert("tonick",new QString(*it));
                sendUserModeChange(channelName.lower(),renderOutput->renderOutput("MODE_USER",replacements));
                ++it;
              }
              break;
            }
            
            case 'n': {
              if (isSet) {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("+n"));
                replacements.insert("setremove",new QString("sets"));
                replacements.insert("tochannel",new QString(channelName));
                sendSetNoMessagesFromOutside(channelName.lower(),true);

              } else {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("-n"));
                replacements.insert("setremove",new QString("removes"));
                replacements.insert("tochannel",new QString(channelName));
                sendSetNoMessagesFromOutside(channelName.lower(),false);
              }
              sendChannelModeChange(channelName.lower(),renderOutput->renderOutput("MODE_CHANNEL",replacements));
              break;
            }
            case 'm': {
              if (isSet) {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("+m"));
                replacements.insert("setremove",new QString("sets"));
                replacements.insert("tochannel",new QString(channelName));
                sendSetModerated(channelName.lower(),true);
              } else {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("-m"));
                replacements.insert("setremove",new QString("removes"));
                replacements.insert("tochannel",new QString(channelName));
                sendSetModerated(channelName.lower(),false);
              }
              sendChannelModeChange(channelName.lower(),renderOutput->renderOutput("MODE_CHANNEL",replacements));
              break;
            }
            case 'a': {
              if (isSet) {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("+a"));
                replacements.insert("setremove",new QString("sets"));
                replacements.insert("tochannel",new QString(channelName));

                sendSetAnonymous(channelName.lower(),true);
              } else {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("-a"));
                replacements.insert("setremove",new QString("removes"));
                replacements.insert("tochannel",new QString(channelName));

                sendSetAnonymous(channelName.lower(),false);

              }
              sendChannelModeChange(channelName.lower(),renderOutput->renderOutput("MODE_CHANNEL",replacements));
              break;
            }
            case 't': {
              if (isSet) {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("+t"));
                replacements.insert("setremove",new QString("sets"));
                replacements.insert("tochannel",new QString(channelName));

                sendSetTopicProtection(channelName.lower(),true);

              } else {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("-t"));
                replacements.insert("setremove",new QString("removes"));
                replacements.insert("tochannel",new QString(channelName));

                sendSetTopicProtection(channelName.lower(),false);

              }
              sendChannelModeChange(channelName.lower(),renderOutput->renderOutput("MODE_CHANNEL",replacements));
              break;
            }
            case 'i': {
              if (isSet) {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("+i"));
                replacements.insert("setremove",new QString("sets"));
                replacements.insert("tochannel",new QString(channelName));
                sendSetInviteOnly(channelName.lower(),true);
              } else {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("-i"));
                replacements.insert("setremove",new QString("removes"));
                replacements.insert("tochannel",new QString(channelName));

                sendSetInviteOnly(channelName.lower(),false);

              }
              sendChannelModeChange(channelName.lower(),renderOutput->renderOutput("MODE_CHANNEL",replacements));
              break;
            }
            case 'q': {
              if (isSet){
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("+q"));
                replacements.insert("setremove",new QString("sets"));
                replacements.insert("tochannel",new QString(channelName));

                sendSetQuite(channelName.lower(),true);

              } else {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("-q"));
                replacements.insert("setremove",new QString("removes"));
                replacements.insert("tochannel",new QString(channelName));

                sendSetQuite(channelName.lower(),false);

              }
              sendChannelModeChange(channelName.lower(),renderOutput->renderOutput("MODE_CHANNEL",replacements));
              break;
            }
            case 'p': {
              if (isSet) {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("+p"));
                replacements.insert("setremove",new QString("sets"));
                replacements.insert("tochannel",new QString(channelName));

                sendSetPrivate(channelName.lower(),true);

              } else {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("-p"));
                replacements.insert("setremove",new QString("removes"));
                replacements.insert("tochannel",new QString(channelName));

                sendSetPrivate(channelName.lower(),false);

              }
              sendChannelModeChange(channelName.lower(),renderOutput->renderOutput("MODE_CHANNEL",replacements));
              break;
            }
            case 's': {
              if (isSet) {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("+s"));
                replacements.insert("setremove",new QString("sets"));
                replacements.insert("tochannel",new QString(channelName));

                sendSetSecret(channelName.lower(),true);

              } else {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("-s"));
                replacements.insert("setremove",new QString("removes"));
                replacements.insert("tochannel",new QString(channelName));

                sendSetSecret(channelName.lower(),false);

              }
              sendChannelModeChange(channelName.lower(),renderOutput->renderOutput("MODE_CHANNEL",replacements));
              break;
            }
            case 'r': {
              if (isSet) {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("+r"));
                replacements.insert("setremove",new QString("sets"));
                replacements.insert("tochannel",new QString(channelName));

                sendSetServerReOp(channelName.lower(),true);

              } else {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("-r"));
                replacements.insert("setremove",new QString("removes"));
                replacements.insert("tochannel",new QString(channelName));

                sendSetServerReOp(channelName.lower(),false);

              }
              sendChannelModeChange(channelName.lower(),renderOutput->renderOutput("MODE_CHANNEL",replacements));
              break;
            }
            case 'l': {
              if (isSet) {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("+l "+*it));
                replacements.insert("setremove",new QString("sets"));
                replacements.insert("tochannel",new QString(channelName));

                sendChannelLimit(channelName.lower(),true,*it);

                ++it;
              } else {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("-l"));
                replacements.insert("setremove",new QString("removes"));
                replacements.insert("tochannel",new QString(channelName));

                sendChannelLimit(channelName.lower(),false,"");

              }
              sendChannelModeChange(channelName.lower(),renderOutput->renderOutput("MODE_CHANNEL",replacements));
              break;
            }
            case 'k': {
              if (isSet) {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("+k "+*it));
                replacements.insert("setremove",new QString("sets"));
                replacements.insert("tochannel",new QString(channelName));

                sendChannelKey(channelName.lower(),true,*it);
                ++it;
              } else {
                replacements.clear();
                replacements.insert("fromnick",new QString(getNickName(from)));
                replacements.insert("mode",new QString("-k"));
                replacements.insert("setremove",new QString("removes"));
                replacements.insert("tochannel",new QString(channelName));
              
                sendChannelKey(channelName.lower(),false,"");

              }
              sendChannelModeChange(channelName.lower(),renderOutput->renderOutput("MODE_CHANNEL",replacements));                                                                                    
              break;
            }
            default: {
              qDebug("Unknown Mode");
              // Send Signal for Plugins later
              break;
            }
          }
        }
      } 
}


/** No descriptions */
const QString & KMILogic::getServerIdent(){
  return serverIdent;
}
/** No descriptions */
void KMILogic::parseUserModes(const QString &from,const QString &modes){
  bool isSet;
  QDict<QString> replacements;
  isSet=false;
  
  for (uint i=0;i<modes.length();i++) {
    switch(modes.at(i).latin1()) {
      case '+':{
        isSet=true;
        break;
      }
      case '-':{
        isSet=false;
        break;
      }
      case 'i':{
        if (isSet) {
          replacements.clear();
          replacements.insert("fromnick",new QString(getNickName(from)));
          replacements.insert("mode",new QString("+i"));
          replacements.insert("setremove",new QString("sets"));
          replacements.insert("tonick",new QString(getNickName(from)));
        } else {
          replacements.clear();
          replacements.insert("fromnick",new QString(getNickName(from)));
          replacements.insert("mode",new QString("-i"));
          replacements.insert("setremove",new QString("removes"));
          replacements.insert("tochannel",new QString(getNickName(from)));
        }
        sendUserModeChange(serverIdent.lower(),renderOutput->renderOutput("MODE_USER",replacements));
        break;
      }
      case 'w': {
        if (isSet) {
          replacements.clear();
          replacements.insert("fromnick",new QString(getNickName(from)));
          replacements.insert("mode",new QString("+w"));
          replacements.insert("setremove",new QString("sets"));
          replacements.insert("tonick",new QString(getNickName(from)));
        } else {
          replacements.clear();
          replacements.insert("fromnick",new QString(getNickName(from)));
          replacements.insert("mode",new QString("-w"));
          replacements.insert("setremove",new QString("removes"));
          replacements.insert("tochannel",new QString(getNickName(from)));
        }
        sendUserModeChange(serverIdent.lower(),renderOutput->renderOutput("MODE_USER",replacements));
        break;
      }

    }
  }
}
