/***************************************************************************
                          returncodes.h  -  description
                             -------------------
    begin                : Don Jun 20 2002
    copyright            : (C) 2002 by Stephan Hermann
    email                : sh@sourcecode.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef RETURNCODES_H
#define RETURNCODES_H

// Client-Server ReturnCodes never send between servers
#define RPL_WELCOME       001    // "Welcome to the Internet Relay Network
                                 // <nick>!<user>@<host>"
#define RPL_YOURHOST      002    // "Your host is <servername>, running version <ver>"
#define RPL_CREATED       003    //  "This server was created <date>"
#define RPL_MYINFO        004    //  "<servername> <version> <available user modes>
                                 //   <available channel modes>"
// AWAY
#define RPL_AWAY          301    // "<nick> :<away message>"
#define RPL_UNAWAY        305    // ":You are no longer marked as being away"
#define RPL_NOWAWAY       306    // ":You have been marked as being away"

// WHOIS
#define RPL_WHOISUSER     311    // "<nick> <user> <host> * :<real name>"
#define RPL_WHOISSERVER   312    // "<nick> <server> :<server info>"
#define RPL_WHOISOPERATOR 313    // "<nick> :is an IRC operator"

#define RPL_WHOISIDLE     317    // "<nick> <integer> :seconds idle"
#define RPL_ENDOFWHOIS    318    // "<nick> :End of /WHOIS list"
#define RPL_WHOISCHANNELS 319    // "<nick> :{[@|+]<channel><space>}"

// WHOWAS
#define RPL_WHOWASUSER    314    // "<nick> <user> <host> * :<real name>"
#define RPL_ENDOFWHOWAS   369    // "<nick> :End of WHOWAS"

// MOTD
#define RPL_MOTDSTART     375    // ":- <server> Message of the day - "
#define RPL_MOTD          372    // ":- <text>"
#define RPL_ENDOFMOTD     376    // ":End of /MOTD command"
// WHO
#define RPL_WHOREPLY      352    // "<channel> <user> <host> <server> <nick>
                                 // ( "H" / "G" > ["*"] [ ( "@" / "+" ) ]
                                 // :<hopcount> <real name>"
#define RPL_ENDOFWHO      315    // "<name> :End of WHO list"

// USERS
#define RPL_USERSSTART    392    // ":UserID   Terminal  Host"
#define RPL_USERS         393    // ":<username> <ttyline> <hostname>"
#define RPL_ENDOFUSERS    394    // ":End of users"
#define RPL_NOUSERS       395    // ":Nobody logged in"

// LUSER
#define RPL_LUSERCLIENT   251    // ":There are <integer> users and <integer> services on <integer> servers"
#define RPL_LUSEROP       252    // "<integer> :operator(s) online"
#define RPL_LUSERUNKNOWN  253    // "<integer> :unknown connection(s)"
#define RPL_LUSERCHANNELS 254    // "<integer> :channels formed"
#define RPL_LUSERME       255    // ":I have <integer> clients and <integer> servers"


// INVITE
#define RPL_INVITING      341    // "nick channel"


// Names
#define RPL_NAMREPLY      353    // "( "=" / "*" / "@" ) <channel>
                                 // :[ "@" / "+" ] <nick> *( " " [ "@" / "+" ] <nick> )
#define RPL_ENDOFNAMES    366    // "<channel> :End of NAMES list"

// TOPIC
#define RPL_NOTOPIC       331    // "<channel> :No topic is set"
#define RPL_TOPIC         332    // "<channel> :<topic>"

// Modes
#define RPL_CHANNELMODEIS 324    // "<channel> <mode> <mode params>"

// VERSION
#define RPL_VERSION       351    // "<version>.<debuglevel> <server> :<comments>"

// LIST
#define RPL_LIST          322    // "<channel> <# visible> :<topic>"
#define RPL_LISTEND       323    // ":End of LIST"

// STATS
#define RPL_STATSLINKINFO 211    // "<linkname> <sendq> <sent messages> <sent Kbytes> <received messages> <received Kbytes> <time open>"
#define RPL_STATSCOMMANDS 212    // "<command> <count> <byte count> <remote count>"
#define RPL_ENDOFSTATS    219    // "<stats letter> :End of STATS report"
#define RPL_STATSUPTIME   242    // ":Server Up %d days %d:%02d:%02d"
#define RPL_STATSOLINE    243    // "O <hostmask> * <name>"

// LINKS
#define RPL_LINKS         364    // "<mask> <server> :<hopcount> <server info>"
#define RPL_ENDOFLINKS    365    // "<mask> :End of LINKS list"

// TIME
#define RPL_TIME          391    // "<server> :<string showing server's local time>"

// TRACE
#define RPL_TRACELINK       200  // "Link <version & debug level> <destination> <next server> V<protocol version> <link uptime in seconds> <backstream sendq> <upstream sendq>"
#define RPL_TRACECONNECTING 201  // "Try. <class> <server>"
#define RPL_TRACEHANDSHAKE  202  // "H.S. <class> <server>"
#define RPL_TRACEUNKNOWN    203  // "???? <class> [<client IP address in dot form>]"
#define RPL_TRACEOPERATOR   204  // "Oper <class> <nick>"
#define RPL_TRACEUSER       205  // "User <class> <nick>"
#define RPL_TRACESERVER     206  // "Serv <class> <int>S <int>C <server> <nick!user|*!*>@<host|server> V<protocol version>"
#define RPL_TRACESERVICE    207  // "Service <class> <name> <type> <active type>"
#define RPL_TRACENEWTYPE    208  // "<newtype> 0 <client name>"
#define RPL_TRACECLASS      209  // "Class <class> <count>"
#define RPL_TRACERECONNECT  210  // Unused
#define RPL_TRACELOG        261  // "File <logfile> <debug level>"
#define RPL_TRACEEND        262  // "<server name> <version & debug level> :End of TRACE"

// ADMIN
#define RPL_ADMINME         256  // "<server> :Administrative info"
#define RPL_ADMINLOC1       257  // ":<admin info>"
#define RPL_ADMINLOC2       258  // ":<admin info>"
#define RPL_ADMINEMAIL      259  // ":<admin info>"

// INFO
#define RPL_INFO            371  // ":<string>"
#define RPL_ENDOFINFO       374  // ":End of INFO list"

// SERVLIST
#define RPL_SERVLIST        234  // "<name> <server> <mask> <type> <hopcount> <info>"
#define RPL_SERVLISTEND     235  // "<mask> <type> :End of service listing"

                        
// dancer IRCD Enhancements


// Error Codes
// implemented
#define ERR_NOSUCHNICK       401   // "<nickname> :No such nick/channel"
#define ERR_NOSUCHSERVER     402   // "<server name> :No such server"
#define ERR_NOSUCHCHANNEL    403   // "<channel name> :No such channel"
#define ERR_CANNOTSENDTOCHAN 404   // "<channel name> :Cannot send to channel"
#define ERR_TOOMANYCHANNELS  405   // "<channel name> :You have joined too many channels"
#define ERR_WASNOSUCHNICK    406   // "<nickname> :There was no such nickname"
#define ERR_TOOMANYTARGETS   407   // "<target> :<error code> recipients. <abort message>"
#define ERR_NOSUCHSERVICE    408   // "<service name> :No such service"
#define ERR_NOORIGIN         409   // ":No origin specified"
#define ERR_NORECIPIENT      411   // ":No recipient given (<command>)"
#define ERR_NOTEXTTOSEND     412   // ":No text to send"
#define ERR_NOTOPLEVEL       413   // "<mask> :No toplevel domain specified"
#define ERR_WILDTOPLEVEL     414   // "<mask> :Wildcard in toplevel domain"
#define ERR_BADMASK          415   // "<mask> :Bad Server/host mask"
#define ERR_UNKNOWNCOMMAND   421   // "<command> :Unknown command"
#define ERR_NOMOTD           422   // ":MOTD File is missing"
#define ERR_NOADMININFO      423   // "<server> :No administrative info available"
#define ERR_FILEERROR        424   // ":File error doing <file op> on <file>"
#define ERR_NONICKNAMEGIVEN  431   // ":No nickname given"
#define ERR_ERRONEUSNICKNAME 432   // "<nick> :Erroneous nickname"
#define ERR_NICKNAMEINUSE    433   // "<nick> :Nickname is already in use"
#define ERR_NICKCOLLISION    436   // "<nick> :Nickname collision KILL from <user>@<host>"
#define ERR_UNAVAILRESOURCE  437   // "<nick/channel> :Nick/channel is temporarily unavailable"
#define ERR_USERNOTINCHANNEL 441   // "<nick> <channel> :They aren't on that channel"
#define ERR_NOTONCHANNEL     442   // "<channel> :You're not on that channel"
#define ERR_USERONCHANNEL    443   // "<user> <channel> :is already on channel"
#define ERR_NOLOGIN          444   // "<user> :User not logged in"
#define ERR_SUMMONDISABLED   445   // ":SUMMON has been disabled"
#define ERR_USERSDISABLED    446   // ":USERS has been disabled"
#define ERR_NOTREGISTERED    451   // ":You have not registered"
#define ERR_NEEDMOREPARAMS   461   // "<command> :Not enough parameters"
#define ERR_ALREADYREGISTRED 462   // ":Unauthorized command (already registered)"
#define ERR_NOPERMFORHOST    463   // ":Your host isn't among the privileged"
#define ERR_PASSWDMISMATCH   464   // ":Password incorrect"
#define ERR_YOUREBANNEDCREEP 465   // ":You are banned from this server"
#define ERR_KEYSET           467   // "<channel> :Channel key already set"
#define ERR_CHANNELISFULL    471   // "<channel> :Cannot join channel (+l)"
#define ERR_UNKNOWNMODE      472   // "<char> :is unknown mode char to me for <channel>"
#define ERR_INVITEONLYCHAN   473   // "<channel> :Cannot join channel (+i)"
#define ERR_BANNEDFROMCHAN   474   // "<channel> :Cannot join channel (+b)"
#define ERR_BADCHANNELKEY    475   // "<channel> :Cannot join channel (+k)"
#define ERR_BADCHANMASK      476   // "<channel> :Bad Channel Mask"
#define ERR_NOCHANMODES      477   // "<channel> :Channel doesn't support modes"
#define ERR_BANLISTFULL      478   // "<channel> <char> :Channel list is full"
#define ERR_NOPRIVILEGES     481   // ":Permission Denied- You're not an IRC operator"
#define ERR_CHANOPRIVSNEEDED 482   // "<channel> :You're not channel operator"
#define ERR_CANTKILLSERVER   483   // ":You can't kill a server!"
#define ERR_RESTRICTED       484 // ":Your connection is restricted!"
#define ERR_UNIQOPPRIVSNEEDED 485  // ":You're not the original channel operator"
#define ERR_NOOPERHOST       491   // ":No O-lines for your host"
#define ERR_UMODEUNKNOWNFLAG 501   // ":Unknown MODE flag"
#define ERR_USERSDONTMATCH   502   // ":Cannot change mode for other users"

// not implemented
#define ERR_YOUWILLBEBANNED  466   //

#endif
